from random import choice
from os import listdir
from os.path import join, isfile, isdir, splitext

from .node import Node
from .pass_file import PassFile

class PassDir(Node):
    def __init__(self, path, name):
        super(PassDir, self).__init__(path, name, None)
        self.dirs = []
        self.files = []

        self.parse_dir(path)


    def parse_dir(self, path):
        for c_name in listdir(path):
            c_path = join(path, c_name)
            name = splitext(c_name)[0]

            if (isdir(c_path) and c_name[0] != '.'):
                self.dirs.append(PassDir(c_path, name))
            elif (isfile(c_path) and c_name[0] != '.'):
                self.files.append(PassFile(c_path, name, None))



import configargparse
import os
import getpass

class Config:
    def __init__(self):
        config = configargparse.ArgParser(
            default_config_files = ['{}/spacepass/config'.format(
                os.getenv('XDG_CONFIG_HOME', '{}/.config'.format(os.path.expanduser("~")))
            )
        ])

        config.add('-c', '--config', required=False, is_config_file=True, help='config file path')

        config.add_argument('-p','--pass-dir', help='Path for the password-store repository')

        config.add_argument('--margin', help='inner margin for the window')
        config.add_argument('--column-spacing', help='Column gutter size')
        config.add_argument('--row-spacing', help='Row gutter size')
        config.add_argument('--max-columns', help='Maximum number of columns to display')
        config.add_argument('--row-height', help='The size for each row')

        self._config = config

    def parse(self):
        self._parsed = self._config.parse()

        if (not self._parsed.pass_dir):
            self._parsed.pass_dir = '{}/.password-store'.format(os.path.expanduser('~'))

        return vars(self._parsed)



import os
from .parser.pass_root import PassRoot
from .gui.dir_window import DirWindow
from .config import Config

def main():
    options = Config().parse()
    content = PassRoot(options['pass_dir'])
    window = DirWindow(options['pass_dir'], content, options)
    window.draw()



from setuptools import setup, find_packages

def readme():
    with open('README.md') as f:
        return f.read()

setup(
    name='spacepass',
    version='0.1a1',
    author='pmcoelho',
    author_email='pmcoelho@protonmail.com',
    url='https://github.com/pm-coelho/spacepass',
    description='A spacemacs(menu) inspired menu for password store',
    long_description=readme(),
    long_description_content_type="text/markdown",
    license='GPLv3+',
    packages=find_packages(exclude=('test')),
    include_package_data=True,
    install_requires=['spacemenu', 'ConfigArgParse'],
    entry_points={
        'console_scripts': ['spacepass=spacepass.run:main']
    },
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: X11 Applications',
        'Environment :: X11 Applications :: GTK',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Operating System :: POSIX :: Linux',
        'Topic :: Security'
    ]
)



from spacemenu.window import Window

# TODO: read options from config
class DirWindow():
    def __init__(self, pass_dir, content, options):
        self.pass_dir = pass_dir
        self._root_window = Window(self._parse_content(content), options)


    def _parse_content(self, content):
        return {
            'label': 'SpacePass',
            'branches': self._parse_dirs(content.dirs),
            'leaves': self._parse_files(content.files)
        }


    def _parse_dirs(self, directories):
        return [
            {
                'label': d.name,
                'branches': self._parse_dirs(d.dirs),
                'leaves': self._parse_files(d.files)
            } for d in directories
        ]


    def _parse_files(self, files):
        return [
            {
                'label': f.name,
                'command': 'pass show -c {}'.format(f.path[len(self.pass_dir)+1:-4])
            } for f in files
        ]


    def draw(self):
        self._root_window.draw()
